import {
  csv,
  range,
  timeParse
} from 'd3';

const allCaps = str => str === str.toUpperCase();
const isRegion = name => allCaps(name) && name != 'WORLD';
export const parseYear = timeParse('%Y');


const melt = (unData, minYear, maxYear ) => {
  const years = range(minYear,maxYear+1);
  const data = [];

  unData.forEach(d => {
    const name = d['Region, subregion, country or area *']
    	.replace('AND THE' ,'&');
    if (isRegion(name)) {
      years.forEach(year => {
        const population = +d[year].replace(/ /g, '') * 1000;
        const row = {
          year: parseYear(year),
          name,
          population
        };
        data.push(row);

      });
    }
  });
  return data;
}

export const getData = () =>
	// Load Multiple Data Files
	Promise
		.all([
      csv('https://vizhub.com/curran/datasets/un-population-estimates-2017-medium-variant.csv'),
      csv('https://vizhub.com/curran/datasets/un-population-estimates-2017.csv')
  	])
		.then(([unDataMedium,unDataEstimate]) => {
      
      //return melt(unDataMedium,2015,2100);
      return melt(unDataEstimate,1950,2014)
      	.concat(melt(unDataMedium,2015,2100));

    }); 
