import {
  select,
  csv, 
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  extent, 
  axisLeft,
  axisBottom,
  line,
  curveBasis,
  nest,
  schemeCategory10,
  descending,
  format,
  mouse
} from 'd3';
import { getData } from './data';
import { lineChart } from './lineChart';
import { colorLegend } from './legend';

// Get the SVG from the DOM
const svg = select('svg');
const lineChartG = svg.append('g');
const colorLegendG = svg.append('g');

// Extract Width and Height from SVG and parse it to float
const height = +svg.attr('height');
const width  = +svg.attr('width');
const margin = {top: 55, right: 195, bottom: 95, left: 115};
 


let selectedYear = 2018;

let data;

const setSelectedYear = year => {
	selectedYear = year;
  render();
};

const render = () => {
  
  const xValue = d => d.year;
  const yValue = d => d.population;
  const colorValue = d => d.name;
  const xAxisLabel = 'Time';
  const yAxisLabel = 'Population';
  const titleLabel = `${yAxisLabel} over ${xAxisLabel} by Region `;
 
  const lastTemp = d => yValue(d.values[d.values.length-1]);

  const nested = nest()
    .key(colorValue)
    .entries(data)
    .sort((a,b ) => descending(lastTemp(a),lastTemp(b)));
   const colorScale = scaleOrdinal(schemeCategory10)
  	.domain(nested.map(d=>d.key));
  
  
   lineChartG
    .call( lineChart,	{
    	data,
     	nested,
  		colorScale,
      colorValue,
      xValue,
      yValue,
      xAxisLabel,
      yAxisLabel,
      titleLabel,
     	margin,
      width,
      height,
     	selectedYear,
     	setSelectedYear
    });

  colorLegendG
    .attr('transform',`translate(780,100)`)
    .call(colorLegend, {
      colorScale,
      itemRadius: 10,
      itemSpacing: 55,
      itemTextPos: 15
  });

};

getData()
  .then(loadedData => {
  	data = loadedData
  	render();
});

