import {
  select,
  csv, 
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  extent, 
  axisLeft,
  axisBottom,
  line,
  curveBasis,
  nest,
  schemeCategory10,
  descending,
  format,
  mouse
} from 'd3';
import { parseYear } from './data';

export const lineChart = (selection, props) => {
  const {
    data,
    nested,
    colorScale,
    colorValue,
    xValue,
    yValue,
    xAxisLabel,
    yAxisLabel,
    titleLabel,
    margin,
    width,
    height,
    selectedYear,
    setSelectedYear
    
  } =props;
  
   const innerWidth = width-margin.left-margin.right;
const innerHeight = height-margin.top-margin.bottom;

  

  
  


  // Set Scales of X and Y Axis
  const xScale = scaleTime()
  	.domain(extent(data, xValue))
  	.range([0,innerWidth]);
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight,0])
  	.nice();
  
 
  // Format X Axis
  const xAxis = axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(20);
  
  // Format Y Axis
  const yAxisTickFormat = number =>
  	format('.2s')(number)
  		.replace('G','B')
  		.replace('.0','');
  const yAxis = axisLeft(yScale)
  	.tickSize(-innerWidth)
  	.tickFormat(yAxisTickFormat)
  	.tickPadding(10);

  // Add Main Bar Group
  const lineG = selection.selectAll('.container').data([null]);
  const lineGEnter = lineG.enter()
  	.append('g')
  		.attr('class', 'container');
  lineGEnter.merge(lineG)
  		.attr('transform', `translate(${margin.left}, ${margin.top})`);

  // Add Y Axis Group
  const yAxisGEnter = lineGEnter
    .append('g')
  		.attr('class','y-axis');
  const yAxisG = lineG.select('.y-axis');
  yAxisGEnter
  	.merge(yAxisG)
    	.call(yAxis)
      .selectAll('.domain')
    		.remove();
  yAxisGEnter
    .append('text')
  		.attr('class','axisLabel')
    	.attr('y',-60)
  		.attr('transform', 'rotate(-90)')
  		.attr('fill', 'black')
    	.attr('text-anchor', 'middle')
  	.merge(yAxisG.select('.axisLabel'))
  		.attr('x',-(innerHeight/2))
  		.text(yAxisLabel);
  
  // Add X Axis Group
  const xAxisGEnter = lineGEnter
    .append('g')
  		.attr('class','x-axis');
  const xAxisG = lineG.select('.x-axis');
  xAxisGEnter
  	.merge(xAxisG)
    	.call(xAxis)
  		.attr('transform', `translate(0, ${innerHeight})`)
      .select('.domain')
  		.remove();
 	xAxisGEnter
    .append('text')
  		.attr('class','axisLabel')
    	.attr('fill', 'black')
  	.merge(xAxisG.select('.axislabel'))
  		.attr('x',innerWidth/2)
  		.attr('y',margin.bottom-10)
  		.text(xAxisLabel);

  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  

  
  // Add The Path
  const linePaths = lineG.merge(lineGEnter)
  	.selectAll('.line-path').data(nested);
  linePaths
  	.enter().append('path')
  		.attr('class','line-path')
  	.merge(linePaths)
  		.attr('d',d => lineGenerator(d.values))
  		.attr('stroke', d => colorScale(d.key));
  
  lineGEnter
    .append('line')
    	.attr('class', 'selected-year-line')
    	.attr('y1', 0)
  	.merge(lineG.selectAll('.selected-year-line'))
      .attr('x1', xScale(parseYear(selectedYear)))
      .attr('x2', xScale(parseYear(selectedYear)))
      .attr('y2', innerHeight)

  

  // Add The Title
  lineGEnter
    .append('text')
			.attr('class','titleLabel')
    	.attr('y', -20)
  	.attr('text-anchor', 'middle')
  	.merge(lineG.selectAll('.titleLabel'))  
      .attr('x',innerWidth/2)
      .text(titleLabel);
  
  lineGEnter
    .append('rect')
  		.attr('class','mouse-interceptor')
      .attr('fill', 'none')
      .attr('pointer-events', 'all')
  	.merge(lineG.selectAll('.mouse-interceptor'))
      .attr('width',innerWidth)
      .attr('height',innerHeight)
      .on('mousemove', function() {
        const x = mouse(this)[0];
        const hoveredDate = xScale.invert(x);
    
    		setSelectedYear(hoveredDate.getFullYear());

      });
  
};